# Generated by Django 4.2.6 on 2024-04-28 06:44

import django.contrib.auth.models
from django.db import migrations
import slth


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
        ('slth', '0005_alter_profile_photo'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
            ],
            options={
                'verbose_name': 'Usuário',
                'verbose_name_plural': 'Usuários',
                'proxy': True,
                'icon': 'users',
                'indexes': [],
                'constraints': [],
            },
            bases=('auth.user', slth.ModelMixin),
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
    ]
