# Generated by Django 4.2.7 on 2024-04-18 16:49

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('slth', '0002_email_role_pushsubscription_error'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Foto',
            new_name='Profile',
        ),
        migrations.AlterModelOptions(
            name='profile',
            options={'verbose_name': 'Profile', 'verbose_name_plural': 'Profile'},
        ),
    ]
