# Generated by Django 4.2.7 on 2024-04-23 09:54

from django.db import migrations
import slth.db.models


class Migration(migrations.Migration):

    dependencies = [
        ('slth', '0004_alter_profile_photo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='photo',
            field=slth.db.models.ImageField(blank=True, null=True, upload_to='', verbose_name='Foto'),
        ),
    ]
